@@ -1,10 +1,10 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-1999, Patrick Powell, San Diego, CA
- *     papowell@astart.com
+ * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ *     papowell@lprng.com
  * See LICENSE for conditions of use.
- * $Id: lpd_status.h,v 5.1 1999/09/12 21:33:03 papowell Exp papowell $
+ * $Id: lpd_status.h,v 5.6 2000/12/25 01:51:21 papowell Exp papowell $
  ***************************************************************************/
 
 
@@ -16,11 +16,13 @@
 int Job_status( int *sock, char *input );
 void Get_queue_status( struct line_list *tokens, int *sock,
 	int displayformat, int status_lines, struct line_list *done_list,
-	int max_size );
-void Print_status_info( int *sock, char *dir, char *file,
+	int max_size, char *hash_key );
+void Print_status_info( int *sock, char *file,
 	char *prefix, int status_lines, int max_size );
+void Print_different_last_status_lines( int *sock, int fd,
+	int status_lines, int max_size );
 void Get_local_or_remote_status( struct line_list *tokens, int *sock,
 	int displayformat, int status_lines, struct line_list *done_list,
-	int max_size );
+	int max_size, char *hash_key );
 
 #endif
