@@ -1,11 +1,16 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-1995 Patrick Powell, San Diego State University
- *     papowell@sdsu.edu
+ * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ *     papowell@lprng.com
  * See LICENSE for conditions of use.
- *
- ***************************************************************************
+ * $Id: portable.h,v 5.17 2000/12/25 01:51:22 papowell Exp papowell $
+ ***************************************************************************/
+
+#ifndef _PLP_PORTABLE_H
+#define _PLP_PORTABLE_H 1
+
+/***************************************************************************
  * MODULE: portable.h
  * PURPOSE:
  * The configure program generates config.h,  which defines various
@@ -22,11 +27,14 @@
  *    Justin Mason <jmason@iona.ie> especially.  Some of the things
  *    that you have to do to get portability are truely bizzare.
  *
- * $Id: portable.h,v 3.2 1996/08/25 22:20:05 papowell Exp papowell $
+ * portable.h,v 3.14 1998/03/24 02:43:22 papowell Exp
  **************************************************************************/
 
-#ifndef _PLP_PORTABLE_H
-#define _PLP_PORTABLE_H 1
+#if !defined(EXTERN)
+#define EXTERN extern
+#define DEFINE(X) 
+#undef DEFS
+#endif
 
 #ifndef __STDC__
 LPRng requires ANSI Standard C compiler
@@ -34,6 +42,10 @@ LPRng requires ANSI Standard C compiler
 
 #include "config.h"
 
+#ifdef HAVE_CTYPE_H
+#include <ctype.h>
+#endif
+
 /*************************************************************************
  * ARGH: some things that "configure" can't get right.
 *************************************************************************/
@@ -54,7 +66,7 @@ LPRng requires ANSI Standard C compiler
  * This appears to be historical.
  *************************************************************************/
 #ifdef apollo
-# define IS_APOLLO
+# define IS_APOLLO OSVERSION
 /* #undef __STDC__ */
 /* # define CONFLICTING_PROTOS */
 #endif
@@ -66,7 +78,7 @@ LPRng requires ANSI Standard C compiler
  * Take a chance on using the standard calls
  *************************************************************************/
 #ifdef ultrix
-# define IS_ULTRIX
+# define IS_ULTRIX OSVERSION
 #endif
 
 
@@ -74,7 +86,7 @@ LPRng requires ANSI Standard C compiler
  * AIX.
  *************************************************************************/
 #ifdef _AIX32 
-# define IS_AIX32
+# define IS_AIX32 OSVERSION
 #endif
 
 /*************************************************************************
@@ -85,49 +97,57 @@ LPRng requires ANSI Standard C compiler
 #endif
 
 
+/*************************************************************************
+ * Cray
+ *************************************************************************/
+
+#if defined(cray)
+#define MAXPATHLEN	1023
+#define HAVE_SIGLONGJMP	1
+
+/* configure incorrectly chooses STATVFS */
+#if defined(USE_STATFS_TYPE)
+#undef  USE_STATFS_TYPE
+#endif
+
+#define USE_STATFS_TYPE	SRV3_STATFS
+#endif
+
+
 /*************************************************************************/
 #if defined(NeXT)
-# define IS_NEXT
+# define IS_NEXT OSVERSION
 # define __STRICT_BSD__
 #endif
 
 /*************************************************************************/
 #if defined(__sgi) && defined(_SYSTYPE_SVR4)
-# define IS_IRIX5
+# define IS_IRIX5 OSVERSION
 #endif
 
 /*************************************************************************/
 #if defined(__sgi) && defined(_SYSTYPE_SYSV)
-#define IS_IRIX4
+#define IS_IRIX4 OSVERSION
 #endif
 
 /*************************************************************************/
 #if defined(__linux__) || defined (__linux) || defined (LINUX)
-# define IS_LINUX
+# define IS_LINUX OSVERSION
 #endif
 
-
-/*************************************************************************/
-#if defined(__hpux) || defined(_HPUX_SOURCE)
-# define IS_HPUX
-# undef _HPUX_SOURCE
-# define _HPUX_SOURCE 1
-#endif
-  
 /*************************************************************************/
 
 #if defined(__convex__) /* Convex OS 11.0 - from w_stef */
-# define IS_CONVEX
+# define IS_CONVEX OSVERSION
 # define LPASS8 (L004000>>16)
 #endif
 
 /*************************************************************************/
 
 #ifdef _AUX_SOURCE
-# define IS_AUX
+# define IS_AUX OSVERSION
 # define _POSIX_SOURCE
 
-# undef HAVE_GETPGRP_0
 # undef SETPROCTITLE
 
 #endif
@@ -135,7 +155,7 @@ LPRng requires ANSI Standard C compiler
 /*************************************************************************/
 
 #if defined(SNI) && defined(sinix)
-# define IS_SINIX
+# define IS_SINIX OSVERSION
 #endif
 
 
@@ -154,7 +174,7 @@ LPRng requires ANSI Standard C compiler
 
 /*************************************************************************/
 #if defined(__bsdi__)
-# define IS_BSDI
+# define IS_BSDI OSVERSION
 #endif
 
 /*************************************************************************/
@@ -210,24 +230,6 @@ LPRng requires ANSI Standard C compiler
  * Patrick Powell Thu Apr  6 07:21:10 PDT 1995
  *********************************************************************/
 
-#include <sys/types.h>
-#include <stdio.h>
-#include <string.h>
-
-#include <sys/param.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#include <arpa/inet.h>
-
-#include <sys/stat.h>
-#include <pwd.h>
-#include <signal.h>
-#include <sys/wait.h>
-#include <ctype.h>
-
-#include <errno.h>
-#include <grp.h>
 /*********************************************************************
  * yuck -- this is a nightmare! half-baked-ANSI systems are poxy (jm)
  *
@@ -255,11 +257,51 @@ LPRng requires ANSI Standard C compiler
   void abort(void);
 #endif
 
+#include <sys/types.h>
+#include <stdio.h>
+#include <string.h>
+#if defined(HAVE_STRINGS_H)
+# include <strings.h>
+#endif
+#include <sys/param.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <netdb.h>
+#include <arpa/inet.h>
+
+#include <sys/stat.h>
+#include <pwd.h>
+#if defined(HAVE_SYS_SIGNAL_H)
+#  include <sys/signal.h>
+#endif
+#include <signal.h>
+#include <sys/wait.h>
+#include <ctype.h>
+
+#include <errno.h>
+#include <grp.h>
+
+#ifdef HAVE_MEMORY_H
+#include <memory.h>
+#endif
+
 #ifndef HAVE_STRCHR
 # define strchr			index
 # define strrchr		rindex
 #endif
 
+/* case insensitive compare for OS without it */
+#if !defined(HAVE_STRCASECMP)
+ int strcasecmp (const char *s1, const char *s2);
+#endif
+#if !defined(HAVE_STRNCASECMP)
+ int strncasecmp (const char *s1, const char *s2, int len );
+#endif
+#if !defined(HAVE_STRCASECMP_DEF)
+ int strcasecmp (const char *s1, const char *s2 );
+#endif
+
+
 /*********************************************************************
  * directory management is nasty.  There are two standards:
  * struct directory and struct dirent.
@@ -320,9 +362,9 @@ typedef struct dirent plp_dir_t;
 # include <time.h>
 #else
 # ifdef HAVE_SYS_TIME_H
-# include <sys/time.h>
-#else
-# include <time.h>
+#   include <sys/time.h>
+# else
+#   include <time.h>
 # endif
 #endif
 
@@ -397,6 +439,8 @@ typedef struct dirent plp_dir_t;
 
 #ifndef HAVE_KILLPG
 # define killpg(pg,sig)	((int) kill ((pid_t)(-(pg)), (sig)))
+#else
+extern int killpg(pid_t pgrp, int sig);
 #endif
 
 /***********************************************************************
@@ -481,6 +525,7 @@ typedef union wait		plp_status_t;
  ***********************************************************************/
 
 typedef RETSIGTYPE plp_signal_t;
+typedef plp_signal_t (*plp_sigfunc_t)(int) ;
 
 #ifndef HAVE_GETDTABLESIZE
 # ifdef NOFILE
@@ -492,21 +537,6 @@ typedef RETSIGTYPE plp_signal_t;
 # endif
 #endif
 
-
-#if !defined(NDEBUG)
-# ifdef HAVE_ASSERT_H
-#  include <assert.h>
-# else
-#  ifdef HAVE_ASSERT
-    extern void assert();
-#  else
-#   define assert(X) if(!(X)){abort(); /* crash and burn */ }
-#  endif
-# endif
-#else
-# define assert(X) /* empty */
-#endif
-
 #ifndef HAVE_STRDUP
 # ifdef __STDC__
    char *strdup(const char*);
@@ -548,8 +578,8 @@ XX ** NO VARARGS ** XX
 #if defined(HAVE_SYS_SYSLOG_H)
 # include <syslog.h>
 #endif
-# if defined(LOG_PID) && defined(LOG_NOWAIT)
-#  define HAVE_OPENLOG
+# if !(defined(LOG_PID) && defined(LOG_NOWAIT) && defined(HAVE_OPENLOG))
+#  undef HAVE_OPENLOG
 # endif /* LOG_PID && LOG_NOWAIT */
 
 /*
@@ -580,7 +610,6 @@ XX ** NO VARARGS ** XX
  * If we have SVR4 and no setpgid() then we need getpgrp() 
  *************************************************************************/
 #if defined(SVR4) || defined(__alpha__)
-# undef HAVE_GETPGRP_0
 # undef HAVE_SETPGRP_0
 #endif
 
@@ -589,22 +618,16 @@ XX ** NO VARARGS ** XX
  * different systems
  */
 
-#ifndef O_NONBLOCK
-#define O_NONBLOCK 0
-#endif
-
-#ifndef O_NOCTTY
-#define O_NOCTTY 0
-#endif
-
-
 #define NONBLOCK (O_NDELAY|O_NONBLOCK)
-#ifdef IS_HPUX
-#undef NONBLOCK
-#define NONBLOCK (O_NONBLOCK)
+#if defined(HPUX) && HPUX<1100
+#  undef NONBLOCK
+#  define NONBLOCK (O_NONBLOCK)
+#  undef FD_SET_FIX
+#  define FD_SET_FIX(X) (int *)
 #endif
 
 
+
 /*********************************************************************
  * AIX systems need this
  *********************************************************************/
@@ -613,4 +636,195 @@ XX ** NO VARARGS ** XX
 # include <sys/select.h>
 #endif
 
-#endif	/* PLP_PORTABLE_H */
+/**********************************************************************
+ *  Signal blocking
+ **********************************************************************/
+#ifdef HAVE_SIGPROCMASK
+/* a signal set */
+#define plp_block_mask sigset_t
+#else
+/* an integer */
+#define plp_block_mask int
+#endif
+
+/**********************************************************************
+ *  Select() problems
+ **********************************************************************/
+#ifdef HAVE_SELECT_H
+#include <select.h>
+#endif
+#if !defined(FD_SET_FIX)
+# define FD_SET_FIX(X) X
+#endif
+
+/**********************************************************************
+ * IPV6 and newer versions
+ **********************************************************************/
+#if !defined(HAVE_INET_PTON)
+int inet_pton( int family, const char *strptr, void *addr );
+#endif
+#if !defined(HAVE_INET_NTOP)
+const char *inet_ntop( int family, const void *addr, char *strptr, size_t len );
+#endif
+
+
+/*****************************************************
+ * Internationalisation of messages, using GNU gettext
+ *****************************************************/
+
+#if HAVE_LOCALE_H
+# include <locale.h>
+#endif
+
+#if ENABLE_NLS
+# include <libintl.h>
+# define _(Text) gettext(Text)
+# ifdef gettext_noop
+#  define N_(Text) gettext_noop(Text)
+# else
+#  define N_(Text) Text
+# endif
+#else
+# define _(Text) Text
+# define N_(Text) Text
+# define textdomain(Domain)
+# define bindtextdomain(Package, Directory)
+#endif
+
+
+/**********************************************************************
+ *  SUNOS Definitions
+ **********************************************************************/
+#ifdef SUNOS
+extern int _flsbuf(int, FILE *);
+extern int _filbuf(FILE *);
+extern int accept(int s, struct sockaddr *name, int *namelen);
+extern int bind(int s, struct sockaddr *name, int namelen);
+extern int connect(int s, struct sockaddr *name, int namelen);
+extern void bzero(void *s, size_t n);
+extern void endgrent( void );
+extern int fflush( FILE *stream );
+extern int fclose( FILE *stream );
+extern int flock( int fd, int operation );
+extern int fprintf(FILE *, const char *, ...);
+extern int fputs( const char *, FILE *stream );
+extern int fstat(int fd, struct stat *buf );
+extern int fseek( FILE *stream, long offset, int ptrname );
+extern int ftruncate( int fd, off_t length );
+extern int fwrite( char *ptr, int size, int nitems, FILE *stream);
+extern int getdtablesize( void );
+extern int getpeername(int s, struct sockaddr *name, int *namelen);
+extern int getsockname(int s, struct sockaddr *name, int *namelen);
+extern int getsockopt(int s, int level, int optname, char *optval,int *optlen);
+extern int ioctl(int fd, int request, caddr_t arg );
+extern int killpg(int pgrp, int sig );
+extern int listen(int s, int backlog );
+extern int lockf(int fd, int cmd, long size );
+/*extern int lseek(int fd, off_t pos, int how ); */
+extern int lstat(const char *path, struct stat *buf );
+#define memmove(dest,src,len) bcopy(src,dest,len)
+extern void bcopy(const void *src,void *dest,size_t len);
+extern int mkstemp(char *s );
+extern int openlog( const char *ident, int logopt, int facility );
+extern int perror(const char *);
+extern int printf( const char *, ...);
+extern int rename(const char *, const char *);
+extern int select (int width, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout);
+extern void setgrent(void);
+extern int seteuid( int euid );
+extern int setreuid( int ruid, int euid );
+extern int setsockopt(int s, int level, int optname, const char *optval,int optlen);
+extern int socket( int domain, int type, int protocol );
+extern int socketpair(int, int, int, int *);
+extern int sscanf( char *s, char *format, ... );
+extern int stat(const char *path, struct stat *buf );
+extern int strcasecmp( const char *, const char * );
+extern char *strerror( int );
+extern int strncasecmp( const char *, const char *, int n );
+extern long strtol( const char *str, char **ptr, int base );
+extern double strtod( const char *str, char **ptr );
+extern int shutdown( int sock, int how );
+extern int gettimeofday(struct timeval *tp, struct timezone *tzp);
+extern int getrlimit(int resource, struct rlimit *rlp);
+extern char * sbrk(int incr);
+extern int fchmod(int fd, int mode);
+extern int strftime(char *buf, int bufsize, const char *fmt, struct tm *tm);
+extern void syslog(int, const char *, ...);
+extern int system( const char *str );
+extern time_t time( time_t *t );
+extern int tolower( int );
+extern int toupper( int );
+extern int tputs( const char *cp, int affcnt, int (*outc)() );
+extern int vfprintf(FILE *, const char *, ...);
+extern int vprintf(FILE *, const char *, va_list ap);
+#endif
+
+
+#ifdef SOLARIS
+extern int setreuid( uid_t ruid, uid_t euid );
+extern int mkstemp(char *s );
+#ifdef HAVE_GETDTABLESIZE
+extern int getdtablesize(void);
+#endif
+#endif
+
+#if !defined(HAVE_SYSLOG_DEF)
+extern void syslog(int, const char *, ...);
+#endif
+#if !defined(HAVE_OPENLOG_DEF)
+extern int openlog( const char *ident, int logopt, int facility );
+#endif
+
+#ifdef IS_AIX32
+extern int seteuid(uid_t);
+#endif
+
+
+/* IPV6 structures define */
+
+#if defined(AF_INET6)
+# if defined(IN_ADDR6)
+#  define in6_addr in_addr6
+# endif
+#endif
+
+#if defined(HAVE_ARPA_NAMESER_H)
+# include <arpa/nameser.h>
+#endif
+#if defined(HAVE_RESOLV_H)
+# include <resolv.h>
+#endif
+
+#ifdef HAVE_INNETGR
+#if !defined(HAVE_INNETGR_DEF)
+extern int innetgr(const char *netgroup,
+    const char *machine, const char *user, const char *domain);
+#endif
+#endif
+
+
+#define Cast_int_to_voidstar(v) ((void *)(long)(v))
+#define Cast_ptr_to_int(v) ((int)(long)(v))
+#define Cast_ptr_to_long(v) ((long)(v))
+
+/* for testing, set -Wall -Wformat and then make */
+
+#if defined(FORMAT_TEST)
+# define FPRINTF fprintf
+# define PRINTF printf
+# define STDOUT stdout
+# define STDERR stderr
+# define SNPRINTF(X,Y) printf(
+# define VSNPRINTF(X,Y) vprintf(
+# define SETSTATUS(X) printf(
+#else
+# define FPRINTF safefprintf
+# define PRINTF safeprintf
+# define STDOUT 1
+# define STDERR 2
+# define SNPRINTF(X,Y) plp_snprintf(X,Y,
+# define VSNPRINTF(X,Y) plp_vsnprintf(X,Y,
+# define SETSTATUS(X) setstatus(X,
+#endif
+
+#endif
